<?php
declare(strict_types = 1);

/**
 * Gestionnaire d'envoi de courriel.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class Mail
{
	/**
	 * Paramètres (message et en-têtes) des courriels à envoyer.
	 *
	 * @var array
	 */
	public $messages = [];



	/**
	 * Notifications.
	 *
	 * @param string $n
	 *   Type de notification souhaitée.
	 * @param array $albums
	 *   Liste des albums concernés par la notification.
	 * @param int $user_exclude
	 *   Identifiant de l'utilisateur à exclure de la notification.
	 * @param array $infos
	 *   Informations complémentaires pour la notification.
	 * @param array $groups_exclude
	 *   Identifiants des groupes à exclure de la notification.
	 *
	 * @return mixed
	 *   Retourne un tableau des groupes qui seront notifiés,
	 *   ou FALSE en cas d'erreur.
	 */
	public function notify(string $notification, array $albums = [],
	int $user_exclude = 0, array $infos = [], array $groups_exclude = [])
	{
		// Récupération des informations utiles de tous les groupes.
		$sql = "SELECT group_id, group_perms, group_admin FROM {groups} WHERE group_id != 2";
		if (!DB::execute($sql) || !$groups_all = DB::fetchAll())
		{
			return FALSE;
		}

		// Sélection des identifiants de groupes autorisés
		// à être notifiés en fonction de leurs permissions.
		$groups_notify = [];
		foreach ($groups_all as &$i)
		{
			// Groupes à exclure.
			if (in_array($i['group_id'], $groups_exclude))
			{
				continue;
			}

			// Il n'y a pas besoin de vérifier les droits pour le super-admin !
			if ($i['group_id'] == 1)
			{
				$groups_notify[] = 1;
				continue;
			}

			$group_perms = Utility::jsonDecode($i['group_perms']);

			// Notifications pour administrateurs.
			if ($i['group_admin'])
			{
				$groups_notify[] = $i['group_id'];
			}

			// Notifications pour membres.
			elseif ($albums && in_array($notification, ['comment-follow', 'items']))
			{
				$cat_path = "?" . str_repeat(', ?', count($albums) - 1);
				$sql = "SELECT 1
						  FROM {categories} AS cat
						 WHERE %s
						   AND cat_path IN ($cat_path)
						 LIMIT 1";
				$sql = sprintf($sql, SQL::catPerms((int) $i['group_id'],
					$group_perms['perm_list']));
				if (DB::execute($sql, array_values($albums)) && DB::fetchVal())
				{
					$groups_notify[] = $i['group_id'];
				}
			}
		}

		// Type de notification.
		switch ($notification)
		{
			// Nouveau commentaire.
			case 'comment' :
				$vars = ['{GALLERY_TITLE}', '{GALLERY_URL}', '{ITEM_URL}', '{USER_NAME}'];
				$sql_user_alert = '_1____';
				break;

			// Suivi de commentaires.
			case 'comment-follow' :
				$vars = ['{GALLERY_TITLE}', '{GALLERY_URL}', '{ITEM_URL}', '{USER_NAME}'];
				$sql_user_alert = '_____1';
				break;

			// Nouveau commentaire en attente.
			case 'comment-pending' :
				$vars = ['{GALLERY_TITLE}', '{GALLERY_URL}', '{USER_NAME}'];
				$sql_user_alert = '__1___';
				break;

			// Nouveaux fichiers.
			case 'items' :
				$vars = ['{GALLERY_TITLE}', '{GALLERY_URL}', '{USER_NAME}', '{USER_URL}'];
				$sql_user_alert = '___1__';
				break;

			// Nouveaux fichiers en attente.
			case 'items-pending' :
				$vars = ['{GALLERY_TITLE}', '{GALLERY_URL}', '{USER_NAME}', '{USER_URL}'];
				$sql_user_alert = '____1_';
				break;

			// Inscription.
			case 'registration' :
				if (Config::$params['users_registration_valid_admin'])
				{
					$vars = ['{GALLERY_TITLE}', '{GALLERY_URL}', '{USER_NAME}'];
					$notification .= '_pending';
				}
				else
				{
					$vars = ['{GALLERY_TITLE}', '{GALLERY_URL}', '{USER_NAME}', '{USER_URL}'];
				}

				$sql_user_alert = '1_____';
				break;
		}

		$msg_subj = self::_notifyMessageSubject($notification, $vars, $infos);
		$sql_where = "1=1";

		// Suivi de commentaires : récupération de l'identifiant
		// des utilisateurs ayant commenté le fichier.
		if ($notification == 'comment-follow' && Config::$params['users'])
		{
			$sql = "SELECT DISTINCT user_id
					  FROM {comments}
					 WHERE user_id NOT IN (2, ?)
					   AND item_id = ?";
			if (!DB::execute($sql, [$user_exclude, (int) $infos['item_id']])
			|| !$users_id = DB::fetchCol('user_id'))
			{
				return FALSE;
			}
			$sql_where = "user_id IN (" . DB::inInt($users_id) . ")";
		}

		// On récupère l'adresse de courriel de tous
		// les utilisateurs autorisés à être notifié.
		$sql = "SELECT user_email
				  FROM {users}
				 WHERE $sql_where
				   AND user_id NOT IN (2, ?)
				   AND user_alert LIKE '$sql_user_alert'
				   AND user_email LIKE '%@%'
				   AND user_status = '1'
				   AND group_id IN (" . DB::inInt($groups_notify) . ")";
		if (!DB::execute($sql, $user_exclude)
		|| !$emails = DB::fetchCol('user_email'))
		{
			return FALSE;
		}
		sort($emails);

		$this->messages[] =
		[
			'to' => trim(Config::$params['mail_auto_primary_recipient_address']),
			'name' => trim(Config::$params['mail_auto_sender_name']),
			'from' => '',
			'subject' => $msg_subj['subject'],
			'message' => $msg_subj['message'],
			'bcc' => implode(', ', $emails)
		];

		return $groups_notify;
	}

	/**
	 * Envoi un ou plusieurs courriel.
	 *
	 * @return bool
	 */
	public function send(): bool
	{
		if (!$this->messages)
		{
			return TRUE;
		}

		$ok = TRUE;
		foreach ($this->messages as &$i)
		{
			if (!is_array($i))
			{
				continue;
			}
			if (!$this->_phpMail($i))
			{
				$ok = FALSE;
				break;
			}
		}

		return $ok;
	}



	/**
	 * Champ "from".
	 *
	 * @param string $email
	 *
	 * @return string
	 */
	private function _from(string $email): string
	{
		if (Utility::isEmpty($email) || strlen($email) > 250)
		{
			$email = Config::$params['mail_auto_sender_address'];
		}

		$this->_sanitize($email);

		return $email;
	}

	/**
	 * En-têtes du courriel.
	 *
	 * @param array $headers
	 * @param string $bcc
	 * @param string $to
	 *
	 * @return string
	 */
	private function _headers(array $headers, string $bcc = '', string $to = ''): string
	{
		if ($bcc)
		{
			$this->_sanitize($bcc);
			$headers[] = 'Bcc: ' . $bcc;
		}

		if ($to)
		{
			$this->_sanitize($to);
			$headers[] = 'To: ' . $to;
		}

		$headers[] = 'Date: ' . date('r');
		$headers[] = 'MIME-Version: 1.0';
		$headers[] = 'Content-transfer-encoding: 8bit';
		$headers[] = 'Content-type: text/plain; charset=UTF-8';
		$headers[] = 'X-Mailer: ' . System::APP_NAME . '/' . $_SERVER['HTTP_HOST'];

		return implode(PHP_EOL, $headers);
	}

	/**
	 * Prépare le message du courriel.
	 *
	 * @param string $str
	 *
	 * @return string
	 */
	private function _message(string $str): string
	{
		$str = str_replace(["\r\n", "\r", "\n"], ["\n", "\n", PHP_EOL], $str);
		$str = wordwrap($str, 70);

		return $str;
	}

	/**
	 * Retourne le message et le sujet d'une notification
	 * avec remplacement des variables correspondantes.
	 *
	 * @param string $notification
	 *   Type de notification.
	 * @param array $vars
	 *   Variables à remplacer.
	 * @param array $i
	 *   Informations complémentaires pour la notification.
	 *
	 * @return array
	 */
	private function _notifyMessageSubject(string $notification, array &$vars, array &$i): array
	{
		$replace = [];
		foreach ($vars as &$var)
		{
			switch ($var)
			{
				case '{GALLERY_TITLE}' :
					$replace[] = htmlspecialchars(Config::$params['gallery_title']);
					break;

				case '{GALLERY_URL}' :
					$replace[] = GALLERY_HOST . App::getURLGallery();
					break;

				case '{ITEM_URL}' :
					$replace[] = array_key_exists('item_id', $i)
						? GALLERY_HOST . App::getURLGallery('item/' . (int) $i['item_id'])
						: $var;
					break;

				case '{USER_NAME}' :
					$replace[] = array_key_exists('user_name', $i)
						? htmlspecialchars($i['user_name'])
						: $var;
					break;

				case '{USER_URL}' :
					$replace[] = array_key_exists('user_id', $i)
						? GALLERY_HOST . App::getURLGallery('user/' . (int) $i['user_id'])
						: $var;
					break;
			}
		}

		$params = ['message' => '', 'subject' => ''];
		foreach ($params as $k => &$v)
		{
			$v = str_replace(
				$vars,
				$replace,
				Config::$params['mail_notify_'
					. str_replace('-', '_', $notification) . '_' . $k]
			);
		}

		return $params;
	}

	/**
	 * Envoi un courriel par la fonction mail() de PHP.
	 *
	 * @param array $i
	 *	Paramètres du mail.
	 *
	 * @return bool
	 */
	private function _phpMail(array &$i): bool
	{
		if (!function_exists('mail'))
		{
			trigger_error('Function mail() is not available.', E_USER_NOTICE);
			return FALSE;
		}

		$to = $i['to'] ?? '';
		$from = $i['from'] ?? '';
		$name = $i['name'] ?? '';
		$bcc = $i['bcc'] ?? '';

		// Doit-on ne pas utiliser le champs "Bcc" ?
		if (Config::$params['mail_auto_bcc'] != '1')
		{
			$to = $bcc;
			$bcc = '';
		}

		// Destinataire(s).
		$to = $this->_to($to);

		// Sujet.
		$subject = $this->_subject($i['subject']);

		// Message.
		$message = $this->_message($i['message']);

		// En-têtes.
		$this->_sanitize($name);
		$name = Utility::isEmpty($name) ? '' : $name . ' ';
		$from = $name . '<' . $this->_from($from) . '>';
		$headers = $this->_headers(['From: ' . $from], $bcc);

		// Envoi.
		$send = mail($to, $subject, $message, $headers);

		return $send;
	}

	/**
	 * Nettoie une chaîne.
	 *
	 * @param string $str
	 *
	 * @return void
	 */
	private function _sanitize(string &$str): void
	{
		$str = Utility::trimAll(
			preg_replace('`(?:to:|b?cc:|from:|content-type:|[\t\r\n\x5C]+)`i', '', $str)
		);
	}

	/**
	 * Prépare le sujet du courriel.
	 *
	 * @param string $str
	 *
	 * @return string
	 */
	private function _subject(string $str): string
	{
		$this->_sanitize($str);

		if (preg_match('`[^\x00-\x3C\x3E-\x7E]`', $str))
		{
			$str = '=?' . CONF_CHARSET . '?B?' . base64_encode($str) . '?=';
		}

		return $str;
	}

	/**
	 * Champ "to".
	 *
	 * @param string $to
	 *
	 * @return string
	 */
	private function _to(string $str): string
	{
		if (Utility::isEmpty($str) || strlen($str) > 250)
		{
			$str = Config::$params['mail_auto_primary_recipient_address'];
		}

		$this->_sanitize($str);

		return $str;
	}
}
?>